Bookk.Utils (bookk v1.0.0)

Copy Markdown View Source

Utility functions.

Summary

Functions

Casts the given value into a Decimal. It raises ArgumentError if the given value is of an unsupported type.

Functions

to_decimal(value)

@spec to_decimal(value) :: Decimal.t() when value: Decimal.t() | integer() | float()

Casts the given value into a Decimal. It raises ArgumentError if the given value is of an unsupported type.